const restrictedGlobals = require('confusing-browser-globals');

const path = require('path');
const fs = require('fs');

const appDirectory = fs.realpathSync(process.cwd());
const resolveApp = relativePath => path.resolve(appDirectory, relativePath);
const projectRootPath = resolveApp('.');
const tsConfigPath = resolveApp('tsconfig.json');


module.exports = {
  parserOptions: {
    project: tsConfigPath,
    tsconfigRootDir: projectRootPath,
    warnOnUnsupportedTypeScriptVersion: true,
  },
  settings: {
    'import/parsers': {
      '@typescript-eslint/parser': ['.ts', '.tsx']
    },
    'import/resolver': {
      // use <root>/tsconfig.json
      typescript: {}
    }
  },
  rules: {
    /**
     * @bug https://github.com/benmosher/eslint-plugin-import/issues/1282
     * "import/named" temporary disable.
     */
    'import/named': 'off',
    'import/export': 'off', // "import/export" temporary disable.
    'import/prefer-default-export': 'off', // Allow single Named-export
    'spaced-comment': 'off', // Spaced comment has to be disabled for reference
    'no-unused-expressions': [
      'warn',
      {
        allowShortCircuit: true,
        allowTernary: true
      }
    ], // https://eslint.org/docs/rules/no-unused-expressions
    // These ESLint rules are known to cause issues with typescript-eslint
    // See https://github.com/typescript-eslint/typescript-eslint/blob/master/packages/eslint-plugin/src/configs/recommended.json
    camelcase: 'off',
    'no-array-constructor': 'off',
    'no-unused-vars': 'off',
    'no-restricted-globals': ['error'].concat(restrictedGlobals),

    /**
     * @description rules of @typescript-eslint
     */
    '@typescript-eslint/interface-name-prefix': ['warn', 'always'],
    '@typescript-eslint/prefer-interface': 'off', // also want to use "type"
    '@typescript-eslint/explicit-function-return-type': 'warn',
    '@typescript-eslint/camelcase': 'warn', // In this case we don't need this to be error
    '@typescript-eslint/no-use-before-define': 'off', // Redundant
    '@typescript-eslint/no-angle-bracket-type-assertion': 'warn',
    '@typescript-eslint/no-array-constructor': 'warn',
    '@typescript-eslint/no-namespace': 'error',
    '@typescript-eslint/indent': 'off', // Let eslint decide
    '@typescript-eslint/no-unused-vars': [
      'error',
      {
        args: 'none',
        ignoreRestSiblings: true,
      },
    ],
    '@typescript-eslint/ban-types': [
      'warn',
      {
        types: {
          Object: {
            message: 'Avoid using the `Object` type. Did you mean `object`?',
            fixWith: 'object'
          },
          Function: 'Avoid using the `Function` type. Prefer a specific function type, like `() => void`.',
          Boolean: {
            message: 'Avoid using the `Boolean` type. Did you mean `boolean`?',
            fixWith: 'boolean'
          },
          Number: {
            message: 'Avoid using the `Number` type. Did you mean `number`?',
            fixWith: 'number'
          },
          Symbol: {
            message: 'Avoid using the `Symbol` type. Did you mean `symbol`?',
            fixWith: 'symbol'
          },
          String: {
            message: 'Avoid using the `String` type. Did you mean `string`?',
            fixWith: 'string'
          }
        }
      }
    ]
  }
}
